import java.util.Vector;
public class Figura {

  private VPontos Vertices;
  private VArestas Arestas;

  private UnionFind Subconjuntos;
  private Vector SemSubconjunto; // booleano

  // Cria uma Figura com 0 vertices e 0 arestas
  public Figura() {
    Vertices = new VPontos(30);
    Arestas = new VArestas(450);
    Subconjuntos = new UnionFind();
    SemSubconjunto = new Vector(30);
  }

  // Devolve o nmero de vrtices.
  public int n() { return Vertices.size(); }
  public int numVertices() { return n(); }

  // Devolve o nmero de arestas da figura.
  public int m() { return Arestas.size(); }
  public int numArestas() { return m(); }

  // Devolve o vrtice de ndice 'i'. Devolve 'null' se o ndice for
  // invlido.
  public Ponto vertice(int i) {
    return Vertices.elementAt(i);
  }

  // Devolve a aresta de ndice 'i'. Devolve 'null' se o ndice for
  // invlido.
  public Aresta aresta(int i) {
    return Arestas.elementAt(i);
  }

  // Insere o vrtice v na figura. Devolve um identificador do vrtice v.
  public int insereVertice(Ponto v) {
    if (!Vertices.contains(v)) {
      Subconjuntos.add();
      Vertices.addElement(v);
      SemSubconjunto.addElement(new Boolean(true));
      return (Vertices.size() - 1);
    }
    return (Vertices.indexOf(v));
  }

  // Move os elementos do [conjunto do vrtice a] para o [conjunto do
  // vrtice e1].
  public void une(int a, int b) {
    Subconjuntos.union(a, b);
    SemSubconjunto.setElementAt(new Boolean(false), a);
    SemSubconjunto.setElementAt(new Boolean(false), b);
System.out.println(Subconjuntos);
  }

  // Devolve o tamanho do subconjuto de v.
  public int tamSubconjunto(int v) {
    return Subconjuntos.size(v);
  }

  // Devolve o representante do subconjuto de v.
  public int subconjunto(int v) {
    return Subconjuntos.find(v);
  }

  // Devolve 'true' se 'v' no pertence a nenhum subconjunto.
  public boolean semSubconjunto(int v) {
    return ((Boolean)SemSubconjunto.elementAt(v)).booleanValue();
  }

  // Devolve o identificador do vrtice v.
  public int idVertice(Ponto v) {
    return (Vertices.indexOf(v));
  }

  // Insere a aresta pq na figura, onde i e j so os ndices dos vrtices
  // p e q.
  public void insereAresta(int i, int j) {
    Aresta a = new Aresta(Vertices.elementAt(i), Vertices.elementAt(j));
    Arestas.addElement(a);
  }

  // Cria todas as arestas na figura.
  public void completa() {
    for (int i = 0; i < Vertices.size(); i++) {
      for (int j = i + 1; j < Vertices.size(); j++) {
        insereAresta(i, j);
      }
    }
  }

  // Devolve true se pq  uma aresta, onde i e j so os ndices dos vrtices
  // p e q. [Se pq  uma aresta, ento qp tambm .]
  public boolean eAresta(int i, int j) {
    return Arestas.contains(new Aresta(Vertices.elementAt(i), 
                                       Vertices.elementAt(j)));
  }

  public void imprime() {
    System.out.println("Pontos: ");
    for (int i = 0; i < Vertices.size(); i++)
      System.out.print(Vertices.elementAt(i) + " ");
    System.out.println();
    System.out.println("Arestas: ");
    for (int i = 0; i < Arestas.size(); i++)
      System.out.print(Arestas.elementAt(i) + " ");
    System.out.println();
  }
}