// projeto de geocomp 2001, IME-USP, julho de 2001

// classe UnionFind
// lku

// Os elementos so representados por inteiros de 0 a (# de elementos - 1).
// Os conjuntos so representados por um elemento.

import java.util.Vector;

public class UnionFind {

  Vector Elementos;

  // Cria um objeto UnionFind com n elementos, cada um em um conjunto
  // diferente.
  public UnionFind(int n) {
    if (n > 5)
      Elementos = new Vector(n);
    else
      Elementos = new Vector();
    for (int i = 0 ; i < n; i++) {
      Elementos.addElement(new Integer(i));
    }
  }
  
  private UnionFind(Vector v) {
    Elementos = v;
  }

  // Cria um objeto UnionFind sem nenhum elemento.
  public UnionFind() {
    Elementos = new Vector();
  }

  // Devolve o nmero de elementos total.
  public int size() {
    return Elementos.size();
  }
  // Devolve o nmero de elementos do conjunto em que est 'e'.
  public int size(int e) {
    int c = ((Integer)(Elementos.elementAt(e))).intValue();
    int n = 0;
    for (int i = 0; i < Elementos.size(); i++)
      if ( ((Integer)(Elementos.elementAt(i))).intValue() == c )
        n++;
    return n;
  }

  // Cria um novo conjunto com um elemento. Devolve um identificador do
  // elemento.
  public int add() {
    int i = Elementos.size();
    Elementos.addElement(new Integer(i));
    return i;
  }

  // Devolve o representante do conjunto a que 'elemento' pertence.
  // Devolve -1 se o elemento no existe.
  public int find(int elemento) {
    try {
      return ((Integer)(Elementos.elementAt(elemento))).intValue();
    }
    catch (ArrayIndexOutOfBoundsException aioobe) {
      return -1;
    }
  }

  // Move os elementos do [conjunto do elemento e2] para o [conjunto do
  // elemento e1].
  public void union(int e1, int e2) {
    try {
      int c1 = ((Integer)(Elementos.elementAt(e1))).intValue();
      int c2 = ((Integer)(Elementos.elementAt(e2))).intValue();
      for (int i = 0; i < Elementos.size(); i++) {
        if ( ((Integer)(Elementos.elementAt(i))).intValue() == c2 )
          Elementos.setElementAt(new Integer(c1),i);
      }
    }
    catch (ArrayIndexOutOfBoundsException aioobe) {
      throw new Error("Erro interno. " + aioobe);
    }
  }

  public void move(int e1, int e2) {
    try {
      int c2 = ((Integer)(Elementos.elementAt(e2))).intValue();
      Elementos.setElementAt(new Integer(c2),e1);
    }
    catch (ArrayIndexOutOfBoundsException aioobe) {
      throw new Error("Erro interno. " + aioobe);
    }
  }
  
  public Object clone() {
    return new UnionFind((Vector)Elementos.clone());
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    for (int i = 0; i < Elementos.size(); i++)
      s.append(i + "." + ((Integer)(Elementos.elementAt(i))) + " ");
    return new String(s);
  }


  /* Teste */
  public static void main(String args[]) {
    UnionFind s = new UnionFind();

    System.out.println(s);
    s.add();s.add(); s.add();s.add();
    s.add();s.add(); s.add();s.add();
    System.out.println(s);

    s.union(1, 2); s.union(3, 7);
    System.out.println(s);
    s.union(2, 7);
    System.out.println(s);

    s.union(2, 1);
    System.out.println(s);
    s.add();s.add(); s.add();s.add();
    System.out.println(s);

    System.out.println(s.size());
    System.out.println(s.size(3));
    System.out.println(s.size(s.find(3)));

  }

}


